// Keehong Youn, 2018
// younkeehong@gmail.com

float t = 0;
float dt = 0.01;

PGraphics o; // 'o' for offscreen
PShader drawing_shader;
PShader sampling_shader;

boolean small_image = false;
boolean follow_mouse = false;
boolean do_blur = false;
int draw_func = 1;

void setup()
{
  size(512, 512, P3D);
  o = createGraphics(width, height, P3D);
  drawing_shader = loadShader("drawing_shader.frag", "drawing_shader.vert");
  sampling_shader = loadShader("sampling_shader.frag", "sampling_shader.vert");
}

void keyPressed()
{
  if (key == 's') small_image = !small_image;
  if (key == 'm') follow_mouse = !follow_mouse;
  if (key == 'b') do_blur = !do_blur;
  if ('0' < key && key <= '9') draw_func = (key - '0');
  if (key == 'r') {
    drawing_shader = loadShader("drawing_shader.frag", "drawing_shader.vert");
    sampling_shader = loadShader("sampling_shader.frag", "sampling_shader.vert");
  }
}

void draw()
{
  t += dt;

  background(255);

  switch (draw_func) {
    default:
    case 1: draw01(); break;
    case 2: draw02(); break;
    case 3: draw03(); break;
    case 4: draw04(); break;
  }
  
  resetShader();
  fill(255, 0, 255);
  textSize(15);
  text("1: default p5 (no input)", width - 400, height - 180);
  text("2: drawing to offscreen (s, m)", width - 400, height - 160);
  text("3: use fragment shader (s, m, b)", width - 400, height - 140);
  text("4: use vertex shader and fragment shader (s, m, b)", width - 400, height - 120);
  text("func (number 1~4): " + nf(draw_func), width - 400, height - 100);
  text("small image (s): " + (small_image? "true" : "false"), width - 400, height - 80);
  text("follow mouse (m): " + (follow_mouse? "true" : "false"), width - 400, height - 60);
  text("blur (b): " + (do_blur? "true" : "false"), width - 400, height - 40);
}

// default drawing
void draw01()
{
  background(0);

  noStroke();
  fill(255, 255, 0);
  rect(50, 50, 100, 100);

  fill(0, 255, 255);
  rect(200, 50, 100, 100);

  fill(255, 0, 255);
  rect(50, 200, 100, 100);

  fill(255, 255, 255);
  rect(200, 200, 100, 100);
}

// drawing to offscreen
void draw02()
{ 
  o.beginDraw();
  o.background(0);
  
  o.noStroke();
  o.fill(255, 255, 0);
  o.rect(50, 50, 100, 100);

  o.fill(0, 255, 255);
  o.rect(200, 50, 100, 100);

  o.fill(255, 0, 255);
  o.rect(50, 200, 100, 100);

  o.fill(255, 255, 255);
  o.rect(200, 200, 100, 100);

  o.endDraw();

  if (!small_image) {
    if (!follow_mouse) image(o, 0, 0);
    else image(o, mouseX, mouseY);
  }
  else {
    if (!follow_mouse) image(o, 0, 0, width / 2, height / 2);
    else image(o, mouseX, mouseY, width / 2, height / 2);
  }
}

// shader for sampling
void draw03()
{
  o.beginDraw();
  o.background(0);
  
  o.noStroke();
  o.fill(255, 255, 0);
  o.rect(50, 50, 100, 100);

  o.fill(0, 255, 255);
  o.rect(200, 50, 100, 100);

  o.fill(255, 0, 255);
  o.rect(50, 200, 100, 100);

  o.fill(255, 255, 255);
  o.rect(200, 200, 100, 100);
  
  o.endDraw();
  
  shader(sampling_shader);
   sampling_shader.set("blur", do_blur? 1.0 : 0.0);

  if (!small_image) {
    if (!follow_mouse) image(o, 0, 0);
    else image(o, mouseX, mouseY);
  }
  else {
    if (!follow_mouse) image(o, 0, 0, width / 2, height / 2);
    else image(o, mouseX, mouseY, width / 2, height / 2);
  }

}

// vertex shader
void draw04()
{
  
  o.beginDraw();
  o.background(0);
  
  o.shader(drawing_shader);
  drawing_shader.set("time", t);

  o.noStroke();
  o.fill(255, 255, 0);
  o.rect(50, 50, 100, 100);

  o.fill(0, 255, 255);
  o.rect(200, 50, 100, 100);

  o.fill(255, 0, 255);
  o.rect(50, 200, 100, 100);

  o.fill(255, 255, 255);
  o.rect(200, 200, 100, 100);
  
  o.resetShader();
  o.endDraw();
  
  shader(sampling_shader);
   sampling_shader.set("blur", do_blur? 1.0 : 0.0);

  if (!small_image) {
    if (!follow_mouse) image(o, 0, 0);
    else image(o, mouseX, mouseY);
  }
  else {
    if (!follow_mouse) image(o, 0, 0, width / 2, height / 2);
    else image(o, mouseX, mouseY, width / 2, height / 2);
  }
}