// Keehong Youn, 2018
// younkeehong@gmail.com

#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif

uniform sampler2D texture; // texture to sample from

uniform float blur; // 1.0 if blur should be done, 0.0 if not

// coming in from vertex shader
varying vec4 vertTexCoord;

const vec2 rcp_size = vec2(1.0 / 128.0, 1.0 / 128.0);

// sum of all elements is 1.0 (weighted average)
const float kernel[25] = float[25](
  0.003765, 0.015019, 0.023792, 0.015019, 0.003765,
  0.015019, 0.059912, 0.094907, 0.059912, 0.015019,
  0.023792, 0.094907, 0.150342, 0.094907, 0.023792,
  0.015019, 0.059912, 0.094907, 0.059912, 0.015019,
  0.003765, 0.015019, 0.023792, 0.015019, 0.003765
);

void main() {
  vec4 tex_color = texture2D(texture, vertTexCoord.st);

  vec4 blur_color = vec4(0.0);
  for (int i = 0; i < 5; i += 1) {
    float x = float(i - 2);
    for (int j = 0; j < 5; j += 1) {
      int idx = 5 * j + i;
      float y = float(j - 2);
      blur_color += kernel[idx] * texture2D(texture, vertTexCoord.st + rcp_size * vec2(x, y));
    }
  }

  // mix(a, b, k) => (1-k)*a + k*b
  vec4 result = mix(tex_color, blur_color, blur);
  gl_FragColor = result;
}